@@ -44,17 +44,19 @@ SWIFT_DEFAULT_USER_PWD=testing
 SWIFT_MEMBER_USER=tester_member
 SWIFT_MEMBER_USER_PWD=member
 
+SWIFT_CONF_DIR=${SWIFT_CONF_DIR:-/etc/swift}
+
 # Storlets install tunables
 STORLETS_DEFAULT_USER_DOMAIN_ID=${STORLETS_DEFAULT_USER_DOMAIN_ID:-default}
 STORLETS_DEFAULT_PROJECT_DOMAIN_ID=${STORLETS_DEFAULT_PROJECT_DOMAIN_ID:-default}
-STORLETS_DOCKER_DEVICE=${STORLETS_DOCKER_DEVICE:-/home/docker_device}
-STORLETS_DOCKER_BASE_IMG=${STORLETS_DOCKER_BASE_IMG:-ubuntu:20.04}
-STORLETS_DOCKER_BASE_IMG_NAME=${STORLETS_DOCKER_BASE_IMG_NAME:-ubuntu_20.04}
-STORLETS_DOCKER_SWIFT_GROUP_ID=${STORLETS_DOCKER_SWIFT_GROUP_ID:-1003}
-STORLETS_DOCKER_SWIFT_USER_ID=${STORLETS_DOCKER_SWIFT_USER_ID:-1003}
+STORLETS_DOCKER_DEVICE=${STORLETS_DOCKER_DEVICE:-/var/lib/storlets}
+if is_fedora; then
+    STORLETS_DOCKER_BASE_IMG=${STORLETS_DOCKER_BASE_IMG:-quay.io/centos/centos:stream9}
+else
+    STORLETS_DOCKER_BASE_IMG=${STORLETS_DOCKER_BASE_IMG:-ubuntu:22.04}
+fi
 STORLETS_SWIFT_RUNTIME_USER=${STORLETS_SWIFT_RUNTIME_USER:-$USER}
 STORLETS_SWIFT_RUNTIME_GROUP=${STORLETS_SWIFT_RUNTIME_GROUP:-$USER}
-STORLETS_MIDDLEWARE_NAME=storlet_handler
 STORLETS_STORLET_CONTAINER_NAME=${STORLETS_STORLET_CONTAINER_NAME:-storlet}
 STORLETS_DEPENDENCY_CONTAINER_NAME=${STORLETS_DEPENDENCY_CONTAINER_NAME:-dependency}
 STORLETS_LOG_CONTAIER_NAME=${STORLETS_LOG_CONTAIER_NAME:-log}
@@ -122,28 +124,23 @@ function configure_swift_and_keystone_for_storlets {
     fi
 
     # Modify relevant Swift configuration files
-    _generate_swift_middleware_conf
-    _generate_storlet-docker-gateway
-
-    sudo python3 devstack/swift_config.py install /tmp/swift_middleware_conf $STORLETS_SWIFT_RUNTIME_USER
-
-    rm /tmp/swift_middleware_conf
-    rm /tmp/storlet-docker-gateway.conf
+    _modify_swift_conf
+    _generate_gateway_conf
 
     # Create storlet related containers and set ACLs
     start_swift
     _export_swift_os_vars
-    openstack object store account set --property Storlet-Enabled=True
+    swift post --meta "Storlet-Enabled:True"
     swift post --read-acl $SWIFT_DEFAULT_PROJECT:$SWIFT_MEMBER_USER $STORLETS_STORLET_CONTAINER_NAME
     swift post --read-acl $SWIFT_DEFAULT_PROJECT:$SWIFT_MEMBER_USER $STORLETS_DEPENDENCY_CONTAINER_NAME
     swift post $STORLETS_LOG_CONTAIER_NAME
 }
 
 function _install_docker {
-    # TODO: Add other dirstors.
-    # This one is geared towards Ubuntu
-    # See other projects that install docker
-
+    if is_fedora; then
+        # NOTE(tkajinam): install_docker.sh requires the yum command
+        sudo dnf install -y yum
+    fi
     wget http://get.docker.com -O install_docker.sh
     chmod 755 install_docker.sh
     sudo bash -x install_docker.sh
@@ -153,57 +150,66 @@ function _install_docker {
     # containers without sudo
     sudo grep -q docker /etc/group
     if [ $? -ne 0 ]; then
-      sudo groupadd docker
+        sudo groupadd docker
     fi
     add_user_to_group $STORLETS_SWIFT_RUNTIME_USER docker
 
+    # Ensure docker daemon is started
+    start_service docker
     if [ $STORLETS_SWIFT_RUNTIME_USER == $USER ]; then
-      # NOTE(takashi): We need this workaroud because we can't reload
-      #                user-group relationship in bash scripts
-      DOCKER_UNIX_SOCKET=/var/run/docker.sock
-      sudo chown $USER:$USER $DOCKER_UNIX_SOCKET
+        # NOTE(takashi): We need this workaround because we can't reload
+        #                user-group relationship in bash scripts
+        DOCKER_UNIX_SOCKET=/var/run/docker.sock
+        sudo chown $USER:$USER $DOCKER_UNIX_SOCKET
     fi
-
-    # Restart docker daemon
-    restart_service docker
 }
 
 function prepare_storlets_install {
     _install_docker
 
-    if is_ubuntu; then
-      install_package openjdk-${STORLETS_JDK_VERSION}-jdk-headless ant
+    if is_fedora; then
+        install_package java-${STORLETS_JDK_VERSION}-openjdk-devel ant
+        install_package python3 python3-devel
     else
-      die $LINENO "Unsupported distro"
+        install_package openjdk-${STORLETS_JDK_VERSION}-jdk-headless ant
+        install_package python3 python3-dev
     fi
-
-    install_python3
 }
 
 function _generate_jre_dockerfile {
-    PYTHON_PACKAGES="python${PYTHON3_VERSION} python${PYTHON3_VERSION}-dev"
-
-    cat <<EOF > ${TMP_REGISTRY_PREFIX}/repositories/${STORLETS_DOCKER_BASE_IMG_NAME}_jre${STORLETS_JDK_VERSION}/Dockerfile
+    if is_fedora; then
+        JDK_PACKAGE="java-${STORLETS_JDK_VERSION}-openjdk-headless"
+        PYTHON_PACKAGES="python3"
+        cat <<EOF > ${TMP_REGISTRY_PREFIX}/repositories/storlet_engine_image/Dockerfile
+FROM $STORLETS_DOCKER_BASE_IMG
+MAINTAINER root
+RUN dnf install ${PYTHON_PACKAGES} ${JDK_PACKAGE} util-linux-core -y && \
+    dnf clean all
+EOF
+    else
+        JDK_PACKAGE="openjdk-${STORLETS_JDK_VERSION}-jdk-headless"
+        PYTHON_PACKAGES="python3 python3.10"
+        cat <<EOF > ${TMP_REGISTRY_PREFIX}/repositories/storlet_engine_image/Dockerfile
 FROM $STORLETS_DOCKER_BASE_IMG
 MAINTAINER root
-
 RUN apt-get update && \
-    apt-get install ${PYTHON_PACKAGES} openjdk-${STORLETS_JDK_VERSION}-jre-headless -y && \
+    apt-get install ${PYTHON_PACKAGES} ${JDK_PACKAGE} -y && \
     apt-get clean
 EOF
+    fi
 }
 
 function create_base_jre_image {
     echo "Create base jre image"
     sudo docker pull $STORLETS_DOCKER_BASE_IMG
-    mkdir -p ${TMP_REGISTRY_PREFIX}/repositories/"$STORLETS_DOCKER_BASE_IMG_NAME"_jre${STORLETS_JDK_VERSION}
+    mkdir -p ${TMP_REGISTRY_PREFIX}/repositories/storlet_engine_image
     _generate_jre_dockerfile
-    cd ${TMP_REGISTRY_PREFIX}/repositories/"$STORLETS_DOCKER_BASE_IMG_NAME"_jre${STORLETS_JDK_VERSION}
-    sudo docker build -q -t ${STORLETS_DOCKER_BASE_IMG_NAME}_jre${STORLETS_JDK_VERSION} .
+    cd ${TMP_REGISTRY_PREFIX}/repositories/storlet_engine_image
+    sudo docker build -t storlet_engine_image .
     cd -
 }
 
-function _generate_logback_xml {
+function create_logback_xml {
     sudo tee /usr/local/lib/storlets/logback.xml <<EOF >/dev/null
 <configuration>
   <appender name="FILE" class="ch.qos.logback.core.rolling.RollingFileAppender">
@@ -232,29 +238,6 @@ EOF
     sudo chmod 0744 /usr/local/lib/storlets/logback.xml
 }
 
-function _generate_jre_storlet_dockerfile {
-    cat <<EOF > ${TMP_REGISTRY_PREFIX}/repositories/"$STORLETS_DOCKER_BASE_IMG_NAME"_jre${STORLETS_JDK_VERSION}_storlets/Dockerfile
-FROM ${STORLETS_DOCKER_BASE_IMG_NAME}_jre${STORLETS_JDK_VERSION}
-MAINTAINER root
-RUN [ "groupadd", "-g", "$STORLETS_DOCKER_SWIFT_GROUP_ID", "swift" ]
-RUN [ "useradd", "-u" , "$STORLETS_DOCKER_SWIFT_USER_ID", "-g", "$STORLETS_DOCKER_SWIFT_GROUP_ID", "swift" ]
-
-CMD ["prod", "/mnt/channels/factory_pipe", "DEBUG"]
-
-ENTRYPOINT ["/usr/local/libexec/storlets/init_container.sh"]
-EOF
-}
-
-function create_storlet_engine_image {
-    echo "Create Storlet engine image"
-    mkdir -p ${TMP_REGISTRY_PREFIX}/repositories/"$STORLETS_DOCKER_BASE_IMG_NAME"_jre${STORLETS_JDK_VERSION}_storlets
-    _generate_logback_xml
-    _generate_jre_storlet_dockerfile
-    cd ${TMP_REGISTRY_PREFIX}/repositories/"$STORLETS_DOCKER_BASE_IMG_NAME"_jre${STORLETS_JDK_VERSION}_storlets
-    sudo docker build -q -t ${STORLETS_DOCKER_BASE_IMG_NAME}_jre${STORLETS_JDK_VERSION}_storlets .
-    cd -
-}
-
 function install_storlets_code {
     echo "Installing storlets"
     cd $REPO_DIR
@@ -272,6 +255,7 @@ function install_storlets_code {
     # NOTE(takashi): We need --no-deps to avoid enum34 installed in py 2 env,
     #                which causes failure in py3 execution.
     pip_install . -t /usr/local/lib/storlets/python --no-compile --no-deps
+    sudo mkdir -p -m 755 /usr/local/libexec/storlets
     for bin_file in storlets-daemon storlets-daemon-factory ; do
         sudo cp `which ${bin_file}` /usr/local/libexec/storlets/
     done
@@ -286,43 +270,62 @@ function install_storlets_code {
     cd -
 }
 
-function _generate_swift_middleware_conf {
-    cat <<EOF > /tmp/swift_middleware_conf
-[proxy-confs]
-proxy_server_conf_file = ${SWIFT_CONF_DIR}/proxy-server.conf
-storlet_proxy_server_conf_file = ${SWIFT_CONF_DIR}/storlet-proxy-server.conf
+function _modify_swift_conf {
+    local swift_proxy_config
+    swift_proxy_config=${SWIFT_CONF_DIR}/proxy-server.conf
+    iniset ${swift_proxy_config} "filter:storlet_handler" use "egg:storlets#storlet_handler"
+    iniset ${swift_proxy_config} "filter:storlet_handler" execution_server proxy
+    iniset ${swift_proxy_config} "filter:storlet_handler" storlet_container $STORLETS_STORLET_CONTAINER_NAME
+    iniset ${swift_proxy_config} "filter:storlet_handler" storlet_dependency $STORLETS_DEPENDENCY_CONTAINER_NAME
+    iniset ${swift_proxy_config} "filter:storlet_handler" storlet_log $STORLETS_LOG_CONTAIER_NAME
+    iniset ${swift_proxy_config} "filter:storlet_handler" storlet_gateway_module $STORLETS_GATEWAY_MODULE
+    iniset ${swift_proxy_config} "filter:storlet_handler" storlet_gateway_conf $STORLETS_GATEWAY_CONF_FILE
+    iniset ${swift_proxy_config} "filter:storlet_handler" storlet_execute_on_proxy_only $STORLETS_PROXY_EXECUTION_ONLY
 
-[object-confs]
-object_server_conf_files = ${SWIFT_CONF_DIR}/object-server/1.conf
+    local proxy_pipeline
+    proxy_pipeline=$(iniget ${swift_proxy_config} "pipeline:main" pipeline)
+    if ! [[ "${proxy_pipeline}" =~ " storlet_handler copy " ]]; then
+        proxy_pipeline=$(echo "${proxy_pipeline}" | sed "s/ copy / storlet_handler copy /")
+        iniset ${swift_proxy_config} "pipeline:main" pipeline "${proxy_pipeline}"
+    fi
 
-[common-confs]
-storlet_middleware = $STORLETS_MIDDLEWARE_NAME
-storlet_container = $STORLETS_STORLET_CONTAINER_NAME
-storlet_dependency = $STORLETS_DEPENDENCY_CONTAINER_NAME
-#storlet_log = $STORLETS_LOG_CONTAIER_NAME
-storlet_gateway_module = $STORLETS_GATEWAY_MODULE
-storlet_gateway_conf = $STORLETS_GATEWAY_CONF_FILE
-storlet_proxy_execution = $STORLETS_PROXY_EXECUTION_ONLY
-EOF
+    local node_number
+    for node_number in ${SWIFT_REPLICAS_SEQ}; do
+        local swift_obj_config
+        local obj_pipeline
+
+        swift_obj_config=${SWIFT_CONF_DIR}/object-server/${node_number}.conf
+        obj_pipeline=$(iniget ${swift_obj_config} "pipeline:main" pipeline)
+        if ! [[ "${obj_pipeline}" =~ " storlet_handler object-server$" ]]; then
+            obj_pipeline=$(echo "${obj_pipeline}" | sed "s/ object-server$/ storlet_handler object-server/")
+            iniset ${swift_obj_config} "pipeline:main" pipeline "${obj_pipeline}"
+        fi
+
+        iniset ${swift_obj_config} "filter:storlet_handler" use "egg:storlets#storlet_handler"
+        iniset ${swift_obj_config} "filter:storlet_handler" execution_server object
+        iniset ${swift_obj_config} "filter:storlet_handler" storlet_container $STORLETS_STORLET_CONTAINER_NAME
+        iniset ${swift_obj_config} "filter:storlet_handler" storlet_dependency $STORLETS_DEPENDENCY_CONTAINER_NAME
+        iniset ${swift_obj_config} "filter:storlet_handler" storlet_log $STORLETS_LOG_CONTAIER_NAME
+        iniset ${swift_obj_config} "filter:storlet_handler" storlet_gateway_module $STORLETS_GATEWAY_MODULE
+        iniset ${swift_obj_config} "filter:storlet_handler" storlet_gateway_conf $STORLETS_GATEWAY_CONF_FILE
+        iniset ${swift_obj_config} "filter:storlet_handler" storlet_execute_on_proxy_only $STORLETS_PROXY_EXECUTION_ONLY
+    done
 }
 
-function _generate_storlet-docker-gateway {
-    cat <<EOF > /tmp/storlet-docker-gateway.conf
-[DEFAULT]
-storlet_logcontainer = $STORLETS_LOG_CONTAIER_NAME
-cache_dir = $STORLETS_CACHE_DIR
-log_dir = $STORLETS_LOGS_DIR
-script_dir = $STORLETS_SCRIPTS_DIR
-storlets_dir = $STORLETS_STORLETS_DIR
-pipes_dir = $STORLETS_PIPES_DIR
-restart_linux_container_timeout = $STORLETS_RESTART_CONTAINER_TIMEOUT
-storlet_timeout = $STORLETS_RUNTIME_TIMEOUT
-EOF
+function _generate_gateway_conf {
+    iniset ${STORLETS_GATEWAY_CONF_FILE} DEFAULT storlet_logcontainer $STORLETS_LOG_CONTAIER_NAME
+    iniset ${STORLETS_GATEWAY_CONF_FILE} DEFAULT cache_dir $STORLETS_CACHE_DIR
+    iniset ${STORLETS_GATEWAY_CONF_FILE} DEFAULT log_dir $STORLETS_LOGS_DIR
+    iniset ${STORLETS_GATEWAY_CONF_FILE} DEFAULT script_dir $STORLETS_SCRIPTS_DIR
+    iniset ${STORLETS_GATEWAY_CONF_FILE} DEFAULT storlets_dir $STORLETS_STORLETS_DIR
+    iniset ${STORLETS_GATEWAY_CONF_FILE} DEFAULT pipes_dir $STORLETS_PIPES_DIR
+    iniset ${STORLETS_GATEWAY_CONF_FILE} DEFAULT restart_linux_container_timeout $STORLETS_RESTART_CONTAINER_TIMEOUT
+    iniset ${STORLETS_GATEWAY_CONF_FILE} DEFAULT storlet_timeout $STORLETS_RUNTIME_TIMEOUT
 }
 
 function _generate_default_tenant_dockerfile {
     cat <<EOF > ${TMP_REGISTRY_PREFIX}/repositories/"$SWIFT_DEFAULT_PROJECT_ID"/Dockerfile
-FROM ${STORLETS_DOCKER_BASE_IMG_NAME}_jre${STORLETS_JDK_VERSION}_storlets
+FROM storlet_engine_image
 MAINTAINER root
 EOF
 }
@@ -332,7 +335,7 @@ function create_default_tenant_image {
     mkdir -p ${TMP_REGISTRY_PREFIX}/repositories/$SWIFT_DEFAULT_PROJECT_ID
     _generate_default_tenant_dockerfile
     cd ${TMP_REGISTRY_PREFIX}/repositories/$SWIFT_DEFAULT_PROJECT_ID
-    sudo docker build -q -t ${SWIFT_DEFAULT_PROJECT_ID:0:13} .
+    sudo docker build -t ${SWIFT_DEFAULT_PROJECT_ID:0:13} .
     cd -
 }
 
@@ -360,9 +363,11 @@ function install_storlets {
 
     echo "Create Docker images"
     create_base_jre_image
-    create_storlet_engine_image
     create_default_tenant_image
 
+    echo "Create logback xml file"
+    create_logback_xml
+
     echo "Create test configuration file"
     create_test_config_file
 
@@ -376,6 +381,8 @@ function uninstall_storlets {
 
     echo "Cleaning all storlets runtime stuff..."
     sudo rm -fr ${STORLETS_DOCKER_DEVICE}
+    # TODO(tkajinam): Remove config options
+    # TODO(tkajinam): Remove docker containers/images
 }
 
 # Restore xtrace
