@@ -16,7 +16,6 @@
 import threading
 from swiftclient import client as swift_client
 from swiftclient import ClientException
-from nose.plugins.attrib import attr
 from storlets.tools.utils import get_member_auth
 from tests.functional.python import StorletPythonFunctionalTest
 import unittest
@@ -76,12 +75,12 @@ class TestTestStorlet(StorletPythonFunctionalTest):
                 self.url, self.token, 'storletlog', 'test.log',
                 None, None, None, None, headers)
             self.assertEqual(200, resp_headers.get('status'))
-            self.assertEqual('aaa', get_text.read())
+            self.assertEqual(b'aaa', get_text.read())
 
         if op == 'print':
             self.assertEqual(200, get_response_status)
-            self.assertIn('op     print', get_text)
-            self.assertIn('param2     val2', get_text)
+            self.assertIn(b'op     print', get_text)
+            self.assertIn(b'param2     val2', get_text)
 
     def test_print(self):
         self.invoke_storlet("print", False)
@@ -89,10 +88,11 @@ class TestTestStorlet(StorletPythonFunctionalTest):
     def test_crash(self):
         self.invoke_storlet("crash")
 
-    @attr('slow')
     def test_hold(self):
         self.invoke_storlet("hold")
 
+    test_hold.slow = 1
+
     def invoke_storlet_in_parallel(self):
         mythreads = []
 
@@ -106,15 +106,16 @@ class TestTestStorlet(StorletPythonFunctionalTest):
         for t in mythreads:
             t.join()
 
-    @attr('slow')
     def test_parallel_print(self):
         self.invoke_storlet_in_parallel()
 
+    test_parallel_print.slow = 1
+
     def test_storlet_acl_get_fail(self):
         headers = {'X-Run-Storlet': self.storlet_name}
         headers.update(self.additional_headers)
         exc_pattern = '^.*403 Forbidden.*$'
-        with self.assertRaisesRegexp(ClientException, exc_pattern):
+        with self.assertRaisesRegex(ClientException, exc_pattern):
             swift_client.get_object(self.member_url, self.member_token,
                                     self.container, 'test_object',
                                     headers=headers)
@@ -123,7 +124,7 @@ class TestTestStorlet(StorletPythonFunctionalTest):
         headers = {'X-Run-Storlet': self.storlet_name}
         headers.update(self.additional_headers)
         exc_pattern = '^.*403 Forbidden.*$'
-        with self.assertRaisesRegexp(ClientException, exc_pattern):
+        with self.assertRaisesRegex(ClientException, exc_pattern):
             swift_client.get_object(self.member_url, self.member_token,
                                     self.container, 'test_object',
                                     headers=headers)
